source("0_prepare.r")

############################################################################################
# Further data preparations
############################################################################################

# redefine compartments for better balancing

sam[sam[,"compartment"] == "biofilm", "compartment"] <- "water"
sam[sam[,"compartment"] == "sediment", "compartment"] <- "sediment"
sam[sam[,"compartment"] == "effluent", "compartment"] <- "water"

# merge with locations
meta <- merge(sam, loc, by="id_location")

# count samples in data categories after redefinitions
n <- aggregate(list(count=1:nrow(meta)), by=meta[,c("wwtp_impact","compartment")], length)
print(acast(wwtp_impact ~ compartment, data=n, value.var="count", fill=0))

# merge resistome data and metadata
x <- merge(res, meta, by="id_sample")

# add info on target class of drug
x <- merge(x, tar, by="family")

# drop data for unclassified genes
x <- x[x[,"family"] != ".unclassified",]

############################################################################################
# Boxplot for a particular case
############################################################################################

# select a particular gene and compartment, then draw a boxplot
family <- "sul"
compartment <- "sediment"
selection <- which((x[,"family"] == family) & (x[,"compartment"] == compartment))
boxplot(copies_per_16S ~ wwtp_impact, data=x[selection,])

############################################################################################
# All boxplots of potential interest
############################################################################################

# let's do boxplots for all possible combinations of genes and compartments;
# to deal with wide ranges and zeros, we apply a pragmatic transformation;
# because it's many graphics, we save them all in a pdf file

# creates a boxplot for a slice of the data
fn <- function(x) {
  # make sure the input is what it should be
  stopifnot(c("copies_per_16S", "wwtp_impact") %in% names(x))
  fam <- unique(x[,"family"]); stopifnot(length(fam) == 1)
  cmp <- unique(x[,"compartment"]); stopifnot(length(cmp) == 1)
  cls <- unique(x[,"class_of_drug"]); stopifnot(length(cls) == 1)  
  # create plot
  trans <- function(x) {x^0.5}
  boxplot(trans(copies_per_16S) ~ wwtp_impact, data=x, yaxt="n", ylab="",
    col=if (cmp == "water") "steelblue" else "khaki")
  expo <- seq(-6, -1, 1)
  yticks <- sort(c(0, 10^expo, 2*10^expo, 5*10^expo))
  axis(side=2, at=trans(yticks), labels=yticks, las=2)
  mtext(side=4, "copies / 16S rRNA gene copies")
  mtext(side=3, paste0(fam," (",cls,") in ",cmp))
}


# apply function to all slices
pdf(file="../outputs/boxplots.pdf", width=4, height=4)
trash <- by(x, x[,c("compartment", "family")], fn)
graphics.off()

############################################################################################
# Test wether the trends observed in boxplots are significant
############################################################################################

# Considerations for selecting a suitable test:
# 1. The reponse variable is unlikely to be normal distributed (within groups), log-
#    transformation is difficult to apply (because of zeros), and we can't be sure
#    that an identical transformation works for all genes --> we better leave the
#    variable as-is and go for a non-parametric test
# 2. The independent variable (wwtp_impact) which splits the data into groups is
#    ordinary scaled. We could treat these as categories (i.e. nominal data) which
#    leads us to a Kruskal-Wallis test, but this remains somewhat non-satisfactory.
#
# A straightforward solution is to recode the ordinal independent variable (wwtp_impact)
# as numeric values and to perform a rank-based correlation test thereafter. Note that,
# in general (e.g. in the context of model fitting), recoding ordinal into numerical
# data is a bad idea, because intervals are arbitrary. However, as long as we do a
# rank-based analysis only, this doesn't matter (numerical codes can be chosen arbitrarily
# as long as they represent the ordinal ranking properly).

# recodes groups
recode <- function(x) {
  out <- rep(NA, length(x))
  out[x == "extreme"] <- 3
  out[x == "high"] <- 2
  out[x == "moderate"] <- 1
  out[x == "none"] <- 0
  out
}

# performs rank-based correlation test for a slice of the data
fn <- function(x) {
  # make sure the input is what it should be
  stopifnot(c("copies_per_16S", "wwtp_impact") %in% names(x))
  cls <- unique(x[,"family"]); stopifnot(length(cls) == 1)
  cmp <- unique(x[,"compartment"]); stopifnot(length(cmp) == 1)
  # rank-based correlation test
  ct <- cor.test(x=recode(x[,"wwtp_impact"]), y=x[,"copies_per_16S"],
    method="spearman", exact=FALSE)
  data.frame(rho=ct$estimate, p_raw=ct$p.value)
}

# apply to all slices
out <- by(x, x[,c("family","compartment")], fn)

# transform result (being a matrix of data frames) into a single data frame
tmp <- NULL
for (cn in colnames(out)) {
  for (rn in rownames(out)) {
    tmp <- rbind(tmp, data.frame(compartment=cn, family=rn, out[rn,cn]))
  }
}
out <- tmp
rm(tmp)

# adjust p-values; we do this separately for the two compartments which
# reflects two independent hypothesis
out <- cbind(out, p_adj=NA)
for (co in unique(out[,"compartment"])) {
  i <- (out[,"compartment"] == co)
  out[i,"p_adj"] <- p.adjust(out[i,"p_raw"], method="BH")
}

# add columns with a symbol-based summary for easy visualization
stars <- function(p) {
  x <- rep("n.s.", length(p))
  x[p < 0.05] <- "*"
  x[p < 0.01] <- "**"
  x[p < 0.001] <- "***"
  x
}
out <- cbind(out, rho_symbol=ifelse(out[,"rho"] <= 0, "-", "+"))
out <- cbind(out, p_raw_symbol=stars(out[,"p_raw"]))
out <- cbind(out, p_adj_symbol=stars(out[,"p_adj"]))

# export as table
write.table(out, file="../outputs/corTest_after_recoding.txt", sep="\t",
  col.names=TRUE, row.names=FALSE, quote=FALSE)



